package com.muyu.mqtt;


import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.github.benmanes.caffeine.cache.Cache;
import com.github.benmanes.caffeine.cache.Caffeine;
import com.muyu.mqtt.dao.MessageDa;
import com.muyu.utils.ConversionUtil;
import lombok.extern.log4j.Log4j2;
import org.apache.iotdb.rpc.IoTDBConnectionException;
import org.apache.iotdb.rpc.StatementExecutionException;
import org.apache.kafka.clients.producer.ProducerRecord;
import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttMessage;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.kafka.core.KafkaAdmin;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Service;

import java.rmi.ServerException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;


/**
 * 回执消息类 MessageCallbackService
 *
 * @author Yangle
 * Date 2024/6/6 15:08
 */
@Service
@Log4j2
public class MessageCallbackService implements MqttCallback {

    @Autowired
    private KafkaTemplate kafkaTemplate;
    @Autowired
    private KafkaAdmin kafkaAdmin;

    @Override
    public void connectionLost(Throwable cause) {
        System.out.println("connectionLost:" + cause.getMessage());
    }


    private static final AtomicInteger PARTITION_COUNTER = new AtomicInteger(0); // 假设NUM_PARTITIONS是已知的分区数量
    private static final int NUM_PARTITIONS = 8; // 请根据实际情况设置分区数量

    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @Override
    public void messageArrived(String topic, MqttMessage message) {
        // 将消息内容从字节数组转换为字符串
        String s = new String(message.getPayload());

        try {
            String s1 = ConversionUtil.hexStringToString(s);
            JSONObject object = getObject(s1);
            log.error("object:{}", object);
            String vin ="";
            String key = vin; // 使用vin作为key，如果适用
            String value = com.alibaba.fastjson.JSON.toJSONString(object);
            log.error("value:{}", value);
            ProducerRecord<String, String> producerRecord = new ProducerRecord<>(topic, key, value);
            kafkaTemplate.send(producerRecord);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken token) {
        System.out.println("deliveryComplete---------" + token.isComplete());
    }

    public com.alibaba.fastjson2.JSONObject getObject(String message) {
        log.info("原始数据:{}", message);
        // 移除字符串message的首尾字符，为后续处理准备
        message = message.substring(1, message.length() - 2);
        // 初始化StringBuilder用于构建最终的JSON字符串
        StringBuffer buffer = new StringBuffer();
        // 从Redis中获取指定key的值，这里的key是message的前17个字符
        Object o = redisTemplate.opsForHash().get("VIN", message.substring(0, 17));
        // 将获取到的值转换为VehicleDateModel的列表
        List<MessageDa> messageDas = new ArrayList<>();
        messageDas = JSON.parseObject(o.toString(), ArrayList.class).stream()
                .map(obj -> JSON.parseObject(obj.toString(), MessageDa.class)).toList();
        // 用于构建最终JSON字符串的变量
        String finalMessage = message;
        // 遍历车辆数据模型列表，构建JSON字符串的键值对
        messageDas.forEach(messageDa -> {
            // 根据车辆数据模型的起始和结束位置，以及特定的处理逻辑，构建JSON字符串的键值对并追加到buffer中
            buffer.append(",\"" + messageDa.getAnalyzeKey() + "\":" +
                    "\"" + remove0(finalMessage.substring(messageDa.getAnalyzeStart() - 1, messageDa.getEnt())) + "\"");
        });
        // 构建最终的JSON字符串
        String jsonString = "{" + buffer.substring(1) + "}";
        com.alibaba.fastjson2.JSONObject jsonObject = JSON.parseObject(jsonString);
        try {

        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        // 日志记录解析后的数据
        log.info("解析后的数据:{}", jsonString);

        // 返回构建好的JSONObject
        return new com.alibaba.fastjson2.JSONObject(jsonObject);
    }

    // 移除字符串中的前导零
    public String remove0(String str) {
        if (str.length() > 1) {
            if (str.charAt(0) == '0') {
                return remove0(str.substring(1));
            } else {
                return str;
            }
        } else {
            return str;
        }
    }

    @RabbitListener(queues = "subscription")
    public void receiveSms(Message message) {
        // 拿到消息中的VIN
        String vin = message.getBody().toString();
        // 删除本地缓存
        localCache.invalidate(vin);
    }

    /**
     * 本地缓存
     */
    Cache<String, String> localCache = Caffeine.newBuilder()
            .initialCapacity(5)
            .maximumSize(10)
            .expireAfterWrite(10, TimeUnit.MINUTES)
            .build();

}
